@@ -116,14 +116,14 @@ enum dump_type {
 
 /* set whole structures!  */
 #define SETUP_FASD_IN(fd) do{ \
-  fas_stream= (fd)->stream->sm.sm_fp; \
+  fas_stream= (fd)->stream; \
   dump_index =   fix((fd)->index) ; \
   current_fasd= * (fd);}while(0)
 
 #define SAVE_CURRENT_FASD \
    struct fasd old_fd; \
    int old_dump_index = dump_index; \
-   FILE *old_fas_stream = fas_stream; \
+   object old_fas_stream = fas_stream; \
    int old_needs_patching = needs_patching; \
    old_fd = current_fasd;
 
@@ -153,7 +153,7 @@ enum dump_type {
 
 #define FASD_VERSION 2
 
-FILE *fas_stream;
+object fas_stream;
 int dump_index;
 struct htent *gethash();
 static void read_fasd1(int i, object *loc);
@@ -164,10 +164,10 @@ object extended_read();
 */   
 /* #define DEBUG */
 
-#ifdef DEBUG
+#ifdef DEBUG /*FIXME debugging versions need sync with getc -> readc_stream, etc.*/
 
-#define PUT(x) putc1((char)x,fas_stream)
-#define GET() getc1()
+#define PUT(x) writec_stream1((char)x,fas_stream)
+#define GET() readc_stream1()
 #define D_FWRITE fwrite1
 #define D_FREAD fread1
 
@@ -236,22 +236,22 @@ char *str;
    {printf("{");
     printf(str,i);
     printf("}");}
- putc(i,fas_stream);}
+ writec_stream(i,fas_stream);}
 
 void
-putc1(x)
+writec_stream1(x)
 int x;
 {  if (debug) printf("(%x,%d,%c)",x,x,x);
-   putc(x,fas_stream);
-   fflush(stdout);
+   writec_stream(x,fas_stream);
+/*    fflush(stdout); */
  }
 
 int
-getc1()
+readc_stream1()
 { int x;
-   x= getc(fas_stream);
+   x= readc_stream(fas_stream);
   if (debug) printf("(%x,%d,%c)",x,x,x);
-  fflush(stdout);
+/*   fflush(stdout); */
   return x;
  }
 
@@ -267,9 +267,9 @@ fread1(p,n1,n2,st)
  {printf("[");
   n1=n1*n2;
   for(i=0;i<n1; i++)
-    putc(p[i],stdout);
+    writec_stream(p[i],sLAstandard_outputA->s.s_dbind);
   printf("]");
-  fflush(stdout);}
+/*   fflush(stdout);} */
     return j;
 
 }
@@ -289,14 +289,14 @@ fwrite1(p,n1,n2,st)
  {printf("[");
   n1=n1*n2;
   for(i=0;i<n1; i++)
-    putc(p[i],stdout);
+    writec_stream(p[i],sLAstandard_outputA->s.s_dbind);
   printf("]");}
     return j;
 }
 
 
-#define GET_OP() (print_op(getc(fas_stream)))
-#define PUT_OP(x) fputc(print_op(x),fas_stream)
+#define GET_OP() ((unsigned)print_op((unsigned char)readc_stream(fas_stream)))
+#define PUT_OP(x) writec_stream(print_op(x),fas_stream)
  
 #define DP(sw)  sw   /*  if (debug) {printf("\ncase sw");} */
 #define GETD(str) getd(str)
@@ -304,7 +304,7 @@ fwrite1(p,n1,n2,st)
 int
 getd(str)
  char *str;
-{ int i = getc(fas_stream);
+{ int i = (unsigned char)readc_stream(fas_stream);
  if(debug){
    printf("{");
    printf(str,i);
@@ -312,16 +312,21 @@ getd(str)
   return i;}
 #define DPRINTF(a,b)  do{if(debug) printf(a,b);} while(0)
 #else
-#define PUT(x) putc((char)x,fas_stream)
-#define GET() getc(fas_stream)
+#define PUT(x) writec_stream((char)x,fas_stream)
+#define GET() ((unsigned char)readc_stream(fas_stream))
 #define GET_OP GET
 #define PUT_OP PUT
-#define D_FWRITE fwrite
-#define D_FREAD SAFE_FREAD
+#define D_FWRITE fwrite_int
+#define D_FREAD fread_int
 #define DP(sw)  sw
 #define PUTD(a,b) PUT(b)
 #define GETD(a) GET()
 #define DPRINTF(a,b)  
+/* #define fwrite_int(a_,b_,c_,d_) {register char *_p=(a_),*_pe=_p+(b_)*(c_);for (;_p<_pe;) writec_stream(*_p++,(d_));} */
+/* #define fread_int(a_,b_,c_,d_)  {register char *_p=(a_),*_pe=_p+(b_)*(c_);for (;_p<_pe;) *_p++=readc_stream(d_);} */
+#define fwrite_int(a_,b_,c_,d_) {register unsigned _i;for (_i=0;_i<(b_)*(c_);_i++) writec_stream(((char *)(a_))[_i],(d_));}
+#define fread_int(a_,b_,c_,d_)  {register unsigned _i;for (_i=0;_i<(b_)*(c_);_i++) ((char *)(a_))[_i]=readc_stream(d_);}
+
 
 #endif
 
@@ -342,7 +347,7 @@ getd(str)
 
 #define MAKE_SHORT(top,bot) (((top)<< SIZE_BYTE) + (bot))
 
-#define READ_BYTE1() getc(fas_stream)
+#define READ_BYTE1() ((unsigned char)readc_stream(fas_stream))
 
 #define GET8(varx ) \
  do{unsigned long var=(unsigned long)READ_BYTE1();  \
@@ -380,7 +385,7 @@ getd(str)
 
 
 #define MASK ~(~0 << 8)
-#define WRITE_BYTEI(x,i)  putc((((x) >> (i*SIZE_BYTE)) & MASK),fas_stream)
+#define WRITE_BYTEI(x,i)  writec_stream((((x) >> (i*SIZE_BYTE)) & MASK),fas_stream)
 
 #define PUTFIX(v_) Join(PUT,SIZEOF_LONG)(v_)
 #define GETFIX(v_) Join(GET,SIZEOF_LONG)(v_)
@@ -512,9 +517,14 @@ do_hash(object obj, int dot)
    }
  
 static void write_fasd(object obj);
-static object
-FFN(write_fasd_top)(object obj, object x)
-{struct fasd *fd = (struct fasd *) x->v.v_self;
+
+DEFUN_NEW("WRITE-FASD-TOP",object,fSwrite_fasd_top,SI
+	  ,2,2,NONE,OO,OO,OO,OO,(object obj, object x),"") {
+
+/* static object */
+/* FFN(write_fasd_top)(object obj, object x) */
+/* { */
+struct fasd *fd = (struct fasd *) x->v.v_self;
   if (fd->direction == sKoutput)
     SETUP_FASD_IN(fd);
   else FEerror("bad value for open slot of fasd",0);
@@ -534,9 +544,12 @@ FFN(write_fasd_top)(object obj, object x)
 #define MAYBE_PATCH(result) \
   if (needs_patching)  result =fasd_patch_sharp(result,0)
 
-static object
-FFN(read_fasd_top)(object x)
-{  struct fasd *fd = (struct fasd *)  x->v.v_self;
+DEFUN_NEW("READ-FASD-TOP",object,fSread_fasd_top,SI
+	  ,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+/* static object */
+/* FFN(read_fasd_top)(object x) */
+/* { */
+  struct fasd *fd = (struct fasd *)  x->v.v_self;
    VOL int e=0;
    object result;
    SAVE_CURRENT_FASD;
@@ -580,60 +593,68 @@ object sLeq;
 object sSPinit;
 void Lmake_hash_table();
 
-static object
-FFN(open_fasd)(object stream, object direction, object eof, object tabl)
-{  object str=Cnil;
-   object result;
-   if(direction==sKinput)
-     {str=coerce_stream(stream,0);
-      if (tabl==Cnil)
-	tabl=alloc_simple_vector(0,aet_object);
-      else
-	check_type(tabl,t_vector);}
-   if(direction==sKoutput)
-     {str=coerce_stream(stream,1);
-      if(tabl==Cnil) tabl=funcall_cfun(Lmake_hash_table,2,sKtest,sLeq);
-      else
-	check_type(tabl,t_hashtable);}
-   check_type(str,t_stream);
-   result=alloc_simple_vector(sizeof(struct fasd)/sizeof(int),aet_object);
-   array_allocself(result,1,Cnil);
-   {struct fasd *fd= (struct fasd *)result->v.v_self;
-    fd->table=tabl;
-    fd->stream=stream;
-    fd->direction=direction;
-    fd->eof=eof;
-    fd->index=small_fixnum(0);
-    fd->package=symbol_value(sLApackageA);
-    fd->filepos = make_fixnum(file_position(stream));
-    
-    SETUP_FASD_IN(fd);
-    if (direction==sKoutput){
-      PUT_OP((int)d_begin_dump);
-      PUTD("version=%d",FASD_VERSION);
-      PUT4(0);  /* reserve space for the size of index array needed */
-          /*  equivalent to:   write_fasd(current_fasd.package);
-	      except we don't want to index this, so that we can open
-	      with an empty array.
-	   */
-      PUT_OP(d_package);
-      write_fasd(current_fasd.package->p.p_name);
+DEFUN_NEW("OPEN-FASD",object,fSopen_fasd,SI
+	  ,4,4,NONE,OO,OO,OO,OO,(object stream, object direction, object eof, object tabl),"") {
 
+
+/* static object */
+/* FFN(open_fasd)(object stream, object direction, object eof, object tabl) */
+/* { */
+  object str=Cnil;
+  object result;
+  if(direction==sKinput)
+    {str=coerce_stream(stream,0);
+    if (tabl==Cnil)
+      tabl=alloc_simple_vector(0,aet_object);
+    else
+      check_type(tabl,t_vector);}
+  if(direction==sKoutput)
+    {str=coerce_stream(stream,1);
+    if(tabl==Cnil) tabl=funcall_cfun(Lmake_hash_table,2,sKtest,sLeq);
+    else
+      check_type(tabl,t_hashtable);}
+  check_type(str,t_stream);
+  result=alloc_simple_vector(sizeof(struct fasd)/sizeof(int),aet_object);
+  array_allocself(result,1,Cnil);
+  {struct fasd *fd= (struct fasd *)result->v.v_self;
+  fd->table=tabl;
+  fd->stream=stream;
+  fd->direction=direction;
+  fd->eof=eof;
+  fd->index=small_fixnum(0);
+  fd->package=symbol_value(sLApackageA);
+  fd->filepos = make_fixnum(file_position(stream));
+  
+  SETUP_FASD_IN(fd);
+  if (direction==sKoutput){
+    PUT_OP((int)d_begin_dump);
+    PUTD("version=%d",FASD_VERSION);
+    PUT4(0);  /* reserve space for the size of index array needed */
+    /*  equivalent to:   write_fasd(current_fasd.package);
+	except we don't want to index this, so that we can open
+	with an empty array.
+    */
+    PUT_OP(d_package);
+    write_fasd(current_fasd.package->p.p_name);
+    
+  }
+  else			/* input */
+    { object tem;
+    read_fasd1(GET_OP(),&tem);
+    if(tem!=current_fasd.table) FEerror("not positioned at beginning of a dump",0);
     }
-    else			/* input */
-      { object tem;
-	read_fasd1(GET_OP(),&tem);
-	if(tem!=current_fasd.table) FEerror("not positioned at beginning of a dump",0);
-      }
-    fd->index=make_fixnum(dump_index);
-    fd->filepos=current_fasd.filepos;
-    fd->package=current_fasd.package;
-    return result;
+  fd->index=make_fixnum(dump_index);
+  fd->filepos=current_fasd.filepos;
+  fd->package=current_fasd.package;
+  return result;
   }}
 
-static object
-FFN(close_fasd)(object ar)
-{  struct fasd *fd= (struct fasd *)(ar->v.v_self);
+DEFUN_NEW("CLOSE-FASD",object,fSclose_fasd,SI
+	  ,1,1,NONE,OO,OO,OO,OO,(object ar),"") {
+/* static object */
+/* FFN(close_fasd)(object ar) */
+/* { */
+  struct fasd *fd= (struct fasd *)(ar->v.v_self);
    check_type(ar,t_vector);
    if (type_of(fd->table)==t_vector)
      /* input uses a vector */
@@ -1054,17 +1075,24 @@ find_sharing(object x)
 	return;
 }
 
-static object
-FFN(find_sharing_top)(object x, object table)
-{sharing_table=table;
- find_sharing(x);
- return Ct;
+/* static object */
+/* FFN(find_sharing_top)(object x, object table) */
+/* {sharing_table=table; */
+/*  find_sharing(x); */
+/*  return Ct; */
+/* } */
+
+DEFUN_NEW("FIND-SHARING-TOP",object,fSfind_sharing_top,SI
+	  ,2,2,NONE,OO,OO,OO,OO,(object x,object table),"") {
+
+  sharing_table=table;
+  find_sharing(x);
+  RETURN1(Ct);
+
 }
 
 
 
-
-
 /* static object            */
 /* read_fasd(int i) */
 /* {object tem; */
@@ -1091,12 +1119,12 @@ lisp_eval(object x)
 
     
 
-#define CHECK_CH(i)    	   do{if ((i)==EOF && feof(fas_stream)) bad_eof();}while (0)
+#define CHECK_CH(i)    	   do{if ((i)==EOF && stream_at_end(fas_stream)) bad_eof();}while (0)
 /* grow vector AR of general type */
 static void
 grow_vector(object ar)
 {   int len=ar->v.v_dim;
-    int nl=(int) (1.5*len);
+    int nl=(int) (1.5*(len+1));
     {BEGIN_NO_INTERRUPT;
      {char *p= (char *)AR_ALLOC(alloc_contblock,nl,object);
     bcopy(ar->v.v_self,p,sizeof(object)* len);
@@ -1496,7 +1524,7 @@ read_fasl_vector(object in)
  object orig = in;
  object d;
  int tem;
- if (((tem=getc(in->sm.sm_fp)) == EOF) && feof(in->sm.sm_fp))
+ if (((tem=(unsigned char)readc_stream(in)) == EOF) && stream_at_end(in))
    { d = coerce_to_pathname(in);
      d = make_pathname(d->pn.pn_host,
 		       d->pn.pn_device,
@@ -1511,16 +1539,16 @@ read_fasl_vector(object in)
        FEerror("Can't open file ~s",1,d);
    }
  else if (tem != EOF)
-   { ungetc(tem,in->sm.sm_fp);}
+   { unreadc_stream(tem,in);}
   while (1)
-   { ch=readc_stream(in);
+   { ch=(unsigned char)readc_stream(in);
      if (ch=='#')
        {unreadc_stream(ch,in);
 	return read_fasl_vector1(in);}
      if (ch== d_begin_dump){
        unreadc_stream(ch,in);
        break;}}
- {object ar=FFN(open_fasd)(in,sKinput,0,Cnil);
+ {object ar=FFN(fSopen_fasd)(in,sKinput,0,Cnil);
   int n=fix(current_fasd.table_length);
   object result,last;
   { BEGIN_NO_INTERRUPT;
@@ -1536,13 +1564,13 @@ read_fasl_vector(object in)
   gset( current_fasd.table->v.v_self,0,n,aet_object);
   END_NO_INTERRUPT;
   }  
-  result=FFN(read_fasd_top)(ar);
+  result=FFN(fSread_fasd_top)(ar);
   if (type_of(result) !=t_vector) goto ERROR;
   last=result->v.v_self[result->v.v_fillp-1];
   if(!consp(last) || last->c.c_car !=sSPinit)
     goto ERROR;
   current_fasd.table->v.v_self = 0;
-  FFN(close_fasd)(ar);
+  FFN(fSclose_fasd)(ar);
   if (orig != in)
     close_stream(in);
   return result;
@@ -1586,10 +1614,10 @@ object IfaslInStream;
 static void
 init_fasdump(void)
 {
-  make_si_sfun("READ-FASD-TOP",read_fasd_top,1);
-  make_si_sfun("WRITE-FASD-TOP",write_fasd_top,2);
-  make_si_sfun("OPEN-FASD",open_fasd,4);  
-  make_si_sfun("CLOSE-FASD",close_fasd,1);
+/*   make_si_sfun("READ-FASD-TOP",read_fasd_top,1); */
+/*   make_si_sfun("WRITE-FASD-TOP",write_fasd_top,2); */
+/*   make_si_sfun("OPEN-FASD",open_fasd,4);   */
+/*   make_si_sfun("CLOSE-FASD",close_fasd,1); */
 /*  make_si_sfun("FASD-I-DATA",fasd_i_macro,1); */
-  make_si_sfun("FIND-SHARING-TOP",find_sharing_top,2);
+/*   make_si_sfun("FIND-SHARING-TOP",find_sharing_top,2); */
 }
