@@ -111,12 +111,6 @@ static object stack_space;
 #  define SIG_STACK_SIZE 1000
 #endif
 
-#ifdef NEED_NONRANDOM_SBRK
-#include <syscall.h>
-#include <sys/personality.h>
-#include <unistd.h>
-#endif
-
 void
 wipe_stack(VOL void *l) {
 
@@ -148,15 +142,8 @@ gcl_main(int argc, char **argv, char **envp)
     struct rlimit rl;
 #endif
 
-#ifdef NEED_NONRANDOM_SBRK
-	{
-	  long pers;
-	  pers=personality(-1);
-	  if (!(pers & ADDR_NO_RANDOMIZE)) {
-	    personality(pers | ADDR_NO_RANDOMIZE);
-	    execve(*argv,argv,envp);
-	  }	
-	}
+#ifdef CAN_UNRANDOMIZE_SBRK
+#include "unrandomize.h"
 #endif
 
 #if defined(DARWIN)
