@@ -65,6 +65,8 @@ gcl_init_init()
   ar_init(gcl_callhash);
   ar_init(gcl_defmacro);
   ar_init(gcl_evalmacros);
+  ar_init(gcl_c);
+  ar_init(gcl_listlib);
   ar_init(gcl_top);
   ar_init(gcl_module);
 
@@ -83,8 +85,6 @@ gcl_init_system(object no_init)
   ar_check_init(gcl_setf,no_init);
   ar_check_init(gcl_arraylib,no_init);
   ar_check_init(gcl_assert,no_init);
-  ar_check_init(gcl_c,no_init);
-  ar_check_init(gcl_listlib,no_init);
   ar_check_init(gcl_defstruct,no_init);
   ar_check_init(gcl_describe,no_init);
 #ifdef HAVE_JAPI_H
@@ -101,6 +101,7 @@ gcl_init_system(object no_init)
   ar_check_init(gcl_packlib,no_init);
   ar_check_init(gcl_seq,no_init);
   ar_check_init(gcl_seqlib,no_init);
+  ar_check_init(gcl_sc,no_init);
   ar_check_init(gcl_trace,no_init);
   ar_check_init(gcl_sloop,no_init);
   ar_check_init(gcl_serror,no_init);
